import jqueryDeprecations from './jqueryDeprecations';

class DrupalDialogEvent extends Event {
  [x: string]: {} | null;
  constructor(
    type: DialogLifeCycleEvents,
    dialog: DrupalDialog,
    settings: DialogOptions,
  ) {
    super(`dialog:${type}`, { bubbles: true });
    this.dialog = dialog;
    this.settings = settings;
  }
}

// In some cases element can be string.
function _createElementFromString(str: string): HTMLElement {
  const tempDiv = document.createElement('div');
  tempDiv.innerHTML = str.trim();
  const el = tempDiv.firstElementChild;
  if (el instanceof HTMLElement) {
    document.body.appendChild(el);
    return el;
  }
  document.body.appendChild(tempDiv);
  return tempDiv;
}

(function (Drupal, drupalSettings) {
  drupalSettings.dialog = {
    autoOpen: true,
    buttonClass: 'button',
    buttonPrimaryClass: 'button--primary',
    close: (event: Event) => {
      const target = event.target as HTMLDialogElement | null;
      target?.dispatchEvent(
        new CustomEvent('dialog-close')
      );
    }
  };

  Drupal.dialog = function (element, options) {
    const settings = { ...drupalSettings.dialog, ...options };

    let domElement: HTMLElement;

    if (typeof element === 'string') {
      domElement = _createElementFromString(element);
    } else {
      domElement = element;
    }

    let dialog = document.createElement('dialog');
    dialog.classList.add('drupal-dialog');
    dialog.setAttribute('draggable', 'true');

    dialog.append(Drupal.theme('dialog', { domElement, ...settings }));
    document.body.append(dialog);

    domElement.dispatchEvent(
      new DrupalDialogEvent('beforecreate', dialog, settings),
    );

    const observer = new MutationObserver((mutationsList) => {
      for (const mutation of mutationsList) {
        if (mutation.attributeName === 'open') {
          if (dialog.hasAttribute('open')) {
            domElement.style.display = 'block';
            setTimeout(() => {
              domElement.dispatchEvent(
                new DrupalDialogEvent('aftercreate', dialog, settings),
              );
            }, 0);
          }
        }
      }
    });

    observer.observe(dialog, { attributes: true });

    const closeDialog = () => {
      domElement.dispatchEvent(
        new DrupalDialogEvent('beforeclose', dialog, settings),
      );

      dialog.close()

      Drupal.detachBehaviors(domElement);

      if (drupalSettings?.dialog?.close) {
        drupalSettings.dialog.close({ ...new Event('closeDialog'), target: domElement });
      }


      setTimeout(() => {
        domElement.dispatchEvent(
          new DrupalDialogEvent('afterclose', dialog, settings),
        );
      }, 0);
    }

    dialog.addEventListener('dialog-close', closeDialog);
    dialog.addEventListener("close", closeDialog);
    dialog.addEventListener('dialog-remove', () => {
      dialog.remove();
    });

    dialog = jqueryDeprecations({
      dialog,
      settings,
      deprecationError: Drupal.deprecationError,
    }) as HTMLDialogElement;

    if (settings.closeOnEscape === false) {
      dialog.addEventListener('cancel', (event) => {
        event.preventDefault();
      });

      dialog.addEventListener('keydown', (event) => {
        if (event.key === 'Escape') {
          event.preventDefault();
        }
      });
    }

    if (settings.autoOpen === true) {
      dialog?.[settings?.modal ? 'showModal' : 'show']();
    }

    const styleProps: (keyof Pick<
      JQueryUI.DialogOptions,
      | 'height'
      | 'width'
      | 'maxHeight'
      | 'maxWidth'
      | 'minHeight'
      | 'minWidth'
      | 'zIndex'
    >)[] = [
        'height',
        'width',
        'maxHeight',
        'maxWidth',
        'minHeight',
        'minWidth',
        'zIndex',
      ];

    styleProps.forEach((prop) => {
      if (settings[prop]) {
        dialog.style[prop === 'zIndex' ? 'zIndex' : prop] = `${settings[prop]}${prop === 'zIndex' ? '' : 'px'
          }`;
      }
    });

    return dialog;
  };
})(window.Drupal, window.drupalSettings);

// Dialog position
// (function (
//   displace,
//   { autoPlacement, autoUpdate, computePosition, offset, size, flip },
// ) {
//   window.addEventListener('dialog:beforecreate', (e) => {
//     const dialogEvent = e as DrupalDialogEvent;
//     const settings = dialogEvent.settings as DialogOptions;
//     const dialog = dialogEvent.dialog as DrupalDialog;
//     if (!settings?.dialogPosition) {
//       return;
//     }
//     dialog.classList.add('drupal-dialog--position');

//     const { reference, options } = settings.dialogPosition;
//     computePosition(reference, dialog, {
//       ...options,
//       middleware: [
//         autoPlacement(),
//         offset(({ rects }) => {
//           return -rects.reference.height / 2 - rects.floating.height / 2;
//         }),
//       ],
//     }).then(({ x, y }) => {
//       Object.assign(dialog.style, {
//         left: `${x}px`,
//         top: `${y}px`,
//       });
//     });
//   });

//   //   // Feature of floating UI.
//   //   // https://floating-ui.com/docs/virtual-elements
//   //   const getVirtualEl = ({ settings }: { settings: JQueryUI.DialogOptions }) => {
//   //     let availableSpace = {
//   //       x: 0,
//   //       y: 0,
//   //       top: 0,
//   //       left: 0,
//   //       bottom: 0,
//   //       right: 0,
//   //       width: window.innerWidth,
//   //       height: window.innerHeight,
//   //     };
//   //     if (!settings?.modal && displace?.offsets) {
//   //       availableSpace = { ...availableSpace, ...displace.offsets };
//   //       availableSpace.width =
//   //         window.innerWidth -
//   //         (displace.offsets.left ?? 0) -
//   //         (displace.offsets.right ?? 0);
//   //       availableSpace.height =
//   //         window.innerHeight -
//   //         (displace.offsets.top ?? 0) -
//   //         (displace.offsets.bottom ?? 0);
//   //     }
//   //     return {
//   //       getBoundingClientRect() {
//   //         return availableSpace;
//   //       },
//   //     };
//   //   };
//   //   const resetPositionFloatingUI = (e) => {
//   //     const widget = e.target?.closest('[role="dialog"]');
//   //     const virtualEl = getVirtualEl(e);
//   //     if (!widget) {
//   //       return;
//   //     }
//   //     const updatePosition = () => {
//   //       computePosition(getVirtualEl(e), widget, {
//   //         strategy: 'fixed',
//   //         middleware: [
//   //           offset(({ rects }) => {
//   //             return {
//   //               mainAxis: -rects.reference.height / 2 - rects.floating.height / 2,
//   //             };
//   //           }),
//   //           size({
//   //             apply({ availableWidth, availableHeight, elements }) {
//   //               Object.assign(elements.floating.style, {
//   //                 maxWidth: `${availableWidth}px`,
//   //                 maxHeight: `${availableHeight}px`,
//   //               });
//   //             },
//   //           }),
//   //         ],
//   //       }).then(({ x, y }) => {
//   //         Object.assign(widget.style, {
//   //           left: `${x}px`,
//   //           top: `${y}px`,
//   //         });
//   //       });
//   //       // Same custom event.
//   //       // https://www.drupal.org/project/drupal/issues/3445033
//   //       e.target?.dispatchEvent(
//   //         new CustomEvent('dialogContentResize', { bubbles: true }),
//   //       );
//   //     };
//   //     // Feature of floating UI.
//   //     // https://floating-ui.com/docs/autoUpdate
//   //     const cleanup = autoUpdate(virtualEl, widget, updatePosition);
//   //     return { cleanup, updatePosition };
//   //   };
//   //   // const { updatePosition } = resetPositionFloatingUI({});
//   //   // window.addEventListener('resize', updatePosition);
// })(window.Drupal.displace, window.FloatingUIDOM);
